@@ -14,6 +14,9 @@
 
 from netaddr import IPAddress
 
+from neutron_lib.api.definitions import bgp as bgp_ext
+from neutron_lib.api.definitions import bgp_4byte_asn
+from neutron_lib.api.definitions import bgp_dragentscheduler as dras_ext
 from neutron_lib.api.definitions import portbindings
 from neutron_lib.callbacks import events
 from neutron_lib.callbacks import registry
@@ -31,12 +34,9 @@ from neutron_dynamic_routing.api.rpc.callbacks import resources as dr_resources
 from neutron_dynamic_routing.api.rpc.handlers import bgp_speaker_rpc as bs_rpc
 from neutron_dynamic_routing.db import bgp_db
 from neutron_dynamic_routing.db import bgp_dragentscheduler_db
-from neutron_dynamic_routing.extensions import bgp as bgp_ext
-from neutron_dynamic_routing.extensions import bgp_4byte_asn
-from neutron_dynamic_routing.extensions import bgp_dragentscheduler as dras_ext
 from neutron_dynamic_routing.services.bgp.common import constants as bgp_consts
 
-PLUGIN_NAME = bgp_ext.BGP_EXT_ALIAS + '_svc_plugin'
+PLUGIN_NAME = bgp_ext.ALIAS + '_svc_plugin'
 LOG = logging.getLogger(__name__)
 
 
@@ -44,9 +44,9 @@ class BgpPlugin(service_base.ServicePluginBase,
                 bgp_db.BgpDbMixin,
                 bgp_dragentscheduler_db.BgpDrAgentSchedulerDbMixin):
 
-    supported_extension_aliases = [bgp_ext.BGP_EXT_ALIAS,
-                                   dras_ext.BGP_DRAGENT_SCHEDULER_EXT_ALIAS,
-                                   bgp_4byte_asn.BGP_4BYTE_ASN_EXT_ALIAS]
+    supported_extension_aliases = [bgp_ext.ALIAS,
+                                   dras_ext.ALIAS,
+                                   bgp_4byte_asn.ALIAS]
 
     def __init__(self):
         super(BgpPlugin, self).__init__()
@@ -60,7 +60,7 @@ class BgpPlugin(service_base.ServicePluginBase,
         self.add_periodic_dragent_status_check()
 
     def get_plugin_type(self):
-        return bgp_ext.BGP_EXT_ALIAS
+        return bgp_ext.ALIAS
 
     def get_plugin_description(self):
         """returns string description of the plugin."""
@@ -76,6 +76,9 @@ class BgpPlugin(service_base.ServicePluginBase,
         return self.conn.consume_in_threads()
 
     def _register_callbacks(self):
+        registry.subscribe(self.floatingip_update_callback,
+                           resources.FLOATING_IP,
+                           events.AFTER_CREATE)
         registry.subscribe(self.floatingip_update_callback,
                            resources.FLOATING_IP,
                            events.AFTER_UPDATE)
@@ -118,10 +121,11 @@ class BgpPlugin(service_base.ServicePluginBase,
     def create_bgp_speaker(self, context, bgp_speaker):
         bgp_speaker = super(BgpPlugin, self).create_bgp_speaker(context,
                                                                 bgp_speaker)
-        payload = {'plugin': self, 'context': context,
-                   'bgp_speaker': bgp_speaker}
-        registry.notify(dr_resources.BGP_SPEAKER, events.AFTER_CREATE,
-                        self, payload=payload)
+        registry.publish(dr_resources.BGP_SPEAKER, events.AFTER_CREATE,
+                         self, payload=events.DBEventPayload(
+                                   context,
+                                   metadata={'plugin': self},
+                                   states=(bgp_speaker,)))
         return bgp_speaker
 
     def update_bgp_speaker(self, context, bgp_speaker_id, bgp_speaker):
@@ -223,19 +227,22 @@ class BgpPlugin(service_base.ServicePluginBase,
         return super(BgpPlugin, self).get_advertised_routes(context,
                                                             bgp_speaker_id)
 
-    def floatingip_update_callback(self, resource, event, trigger, **kwargs):
-        if event != events.AFTER_UPDATE:
+    def floatingip_update_callback(self, resource, event, trigger, payload):
+        if event not in [events.AFTER_CREATE, events.AFTER_UPDATE]:
             return
-
         ctx = context.get_admin_context()
-        new_router_id = kwargs['router_id']
-        last_router_id = kwargs.get('last_known_router_id')
-        floating_ip_address = kwargs['floating_ip_address']
+        new_fip = payload.latest_state
+        new_router_id = new_fip['router_id']
+        floating_ip_address = new_fip['floating_ip_address']
         dest = str(floating_ip_address) + '/32'
         bgp_speakers = self._bgp_speakers_for_gw_network_by_family(
             ctx,
-            kwargs['floating_network_id'],
+            new_fip['floating_network_id'],
             n_const.IP_VERSION_4)
+        last_router_id = None
+        if event == events.AFTER_UPDATE:
+            old_fip = payload.states[0]
+            last_router_id = old_fip['router_id']
 
         if last_router_id and new_router_id != last_router_id:
             # Here gives the old route next_hop a `None` value, then
@@ -255,23 +262,23 @@ class BgpPlugin(service_base.ServicePluginBase,
                                                 bgp_speaker.id,
                                                 [new_host_route])
 
-    def router_interface_callback(self, resource, event, trigger, **kwargs):
+    def router_interface_callback(self, resource, event, trigger,
+                                  payload=None):
         if event == events.AFTER_CREATE:
-            self._handle_router_interface_after_create(**kwargs)
+            self._handle_router_interface_after_create(payload)
         if event == events.AFTER_DELETE:
-            gw_network = kwargs['network_id']
+            gw_network = payload.metadata.get('network_id')
             next_hops = self._next_hops_from_gateway_ips(
-                                                        kwargs['gateway_ips'])
+                payload.metadata.get('gateway_ips'))
             ctx = context.get_admin_context()
             speakers = self._bgp_speakers_for_gateway_network(ctx, gw_network)
             for speaker in speakers:
                 routes = self._route_list_from_prefixes_and_next_hop(
-                                                kwargs['cidrs'],
-                                                next_hops[speaker.ip_version])
+                    payload.metadata['cidrs'], next_hops[speaker.ip_version])
                 self._handle_router_interface_after_delete(gw_network, routes)
 
-    def _handle_router_interface_after_create(self, **kwargs):
-        gw_network = kwargs['network_id']
+    def _handle_router_interface_after_create(self, payload):
+        gw_network = payload.metadata.get('network_id')
         if not gw_network:
             return
 
@@ -280,13 +287,11 @@ class BgpPlugin(service_base.ServicePluginBase,
             speakers = self._bgp_speakers_for_gateway_network(ctx,
                                                               gw_network)
             next_hops = self._next_hops_from_gateway_ips(
-                                                    kwargs['gateway_ips'])
+                payload.metadata.get('gateway_ips'))
 
             for speaker in speakers:
                 prefixes = self._tenant_prefixes_by_router(
-                                                      ctx,
-                                                      kwargs['router_id'],
-                                                      speaker.id)
+                    ctx, payload.resource_id, speaker.id)
                 next_hop = next_hops.get(speaker.ip_version)
                 if next_hop:
                     rl = self._route_list_from_prefixes_and_next_hop(prefixes,
@@ -347,12 +352,12 @@ class BgpPlugin(service_base.ServicePluginBase,
                 self.stop_route_advertisements(ctx, self._bgp_rpc,
                                                speaker.id, routes)
 
-    def port_callback(self, resource, event, trigger, **kwargs):
+    def port_callback(self, resource, event, trigger, payload):
         if event != events.AFTER_UPDATE:
             return
 
-        original_port = kwargs['original_port']
-        updated_port = kwargs['port']
+        original_port = payload.states[0]
+        updated_port = payload.latest_state
         if not updated_port.get('fixed_ips'):
             return
 
